"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _merge2 = _interopRequireDefault(require("lodash/merge"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _mdbid = _interopRequireDefault(require("mdbid"));

var _webinyEntity = require("webiny-entity");

var _webinyModel = require("webiny-model");

var _statements = require("./statements");

var _model = require("./model");

var _operators = _interopRequireDefault(require("./operators"));

class MySQLDriver extends _webinyEntity.Driver {
  constructor(options) {
    super();
    this.operators = (0, _objectSpread2.default)({}, _operators.default, options.operators || {});
    this.connection = options.connection;
    this.model = options.model || _model.MySQLModel;
    this.tables = (0, _objectSpread2.default)({
      prefix: ""
    }, options.tables || {});
    this.autoIncrementIds = options.autoIncrementIds || false;
  }

  setOperator(name, operator) {
    this.operators[name] = operator;
    return this;
  }

  onEntityConstruct(entity) {
    if (this.autoIncrementIds) {
      entity.attr("id").integer().setValidators((value, attribute) => this.isId(attribute.getParentModel().getParentEntity(), value));
    } else {
      entity.attr("id").char().setValidators((value, attribute) => this.isId(attribute.getParentModel().getParentEntity(), value));
    }
  }

  getModelClass() {
    return this.model;
  } // eslint-disable-next-line


  async save(entity, options) {
    if (entity.isExisting()) {
      const data = await entity.toStorage();

      if ((0, _isEmpty2.default)(data)) {
        return new _webinyEntity.QueryResult(true);
      }

      const sql = new _statements.Update({
        operators: this.operators,
        table: this.getTableName(entity),
        data,
        where: {
          id: entity.id
        },
        limit: 1
      }, entity).generate();
      await this.getConnection().query(sql);
      return new _webinyEntity.QueryResult(true);
    }

    if (!this.autoIncrementIds) {
      // If ID was assigned manually, just check if it's valid.
      if (entity.id) {
        if (!this.isId(entity, entity.id)) {
          throw Error(`You have assigned an invalid id (${entity.id})`);
        }
      } else {
        entity.id = MySQLDriver.__generateID();
      }
    }

    const data = await entity.toStorage();
    const sql = new _statements.Insert({
      operators: this.operators,
      data,
      table: this.getTableName(entity)
    }, entity).generate();

    try {
      const results = await this.getConnection().query(sql);

      if (this.autoIncrementIds) {
        entity.id = results.insertId;
      }
    } catch (e) {
      const idAttribute = entity.getAttribute("id");
      idAttribute.reset();
      throw e;
    }

    return new _webinyEntity.QueryResult(true);
  } // eslint-disable-next-line


  async delete(entity, options) {
    const sql = new _statements.Delete({
      operators: this.operators,
      table: this.getTableName(entity),
      where: {
        id: entity.id
      },
      limit: 1
    }, entity).generate();
    await this.getConnection().query(sql);
    return new _webinyEntity.QueryResult(true);
  }

  async find(entity, options) {
    const clonedOptions = (0, _merge2.default)({}, options, {
      operators: this.operators,
      table: this.getTableName(entity),
      operation: "select",
      limit: 10,
      offset: 0
    });

    MySQLDriver.__preparePerPageOption(clonedOptions);

    MySQLDriver.__preparePageOption(clonedOptions);

    MySQLDriver.__prepareQueryOption(clonedOptions);

    MySQLDriver.__prepareSearchOption(clonedOptions);

    clonedOptions.calculateFoundRows = true;
    const sql = new _statements.Select(clonedOptions, entity).generate();
    const results = await this.getConnection().query(sql);
    const totalCount = await this.getConnection().query("SELECT FOUND_ROWS() as count");
    const meta = (0, _webinyEntity.createPaginationMeta)({
      totalCount: totalCount[0].count,
      page: options.page,
      perPage: options.perPage
    });
    return new _webinyEntity.QueryResult(results, meta);
  }

  async findOne(entity, options) {
    const clonedOptions = {
      operators: this.operators,
      table: this.getTableName(entity),
      where: options.query,
      search: options.search,
      groupBy: options.groupBy,
      sort: options.sort,
      sql: options.sql,
      page: options.page,
      limit: 1
    };

    MySQLDriver.__preparePerPageOption(clonedOptions);

    MySQLDriver.__preparePageOption(clonedOptions);

    MySQLDriver.__prepareQueryOption(clonedOptions);

    MySQLDriver.__prepareSearchOption(clonedOptions);

    const sql = new _statements.Select(clonedOptions, entity).generate();
    const results = await this.getConnection().query(sql);
    return new _webinyEntity.QueryResult(results[0]);
  }

  async count(entity, options) {
    const clonedOptions = (0, _merge2.default)({}, options, {
      operators: this.operators,
      table: this.getTableName(entity),
      columns: ["COUNT(*) AS count"]
    }, entity);

    MySQLDriver.__prepareQueryOption(clonedOptions);

    MySQLDriver.__prepareSearchOption(clonedOptions);

    const sql = new _statements.Select(clonedOptions, entity).generate();
    const results = await this.getConnection().query(sql);
    return new _webinyEntity.QueryResult(results[0].count);
  } // eslint-disable-next-line


  isId(entity, value, options) {
    if (this.autoIncrementIds) {
      return typeof value === "number" && Number.isInteger(value) && value > 0;
    }

    if (typeof value === "string") {
      return value.match(new RegExp("^[0-9a-fA-F]{24}$")) !== null;
    }

    return false;
  }

  getConnection() {
    return this.connection;
  }

  setTablePrefix(tablePrefix) {
    this.tables.prefix = tablePrefix;
    return this;
  }

  getTablePrefix() {
    return this.tables.prefix;
  }

  setTableNaming(tableNameValue) {
    this.tables.naming = tableNameValue;
    return this;
  }

  getTableNaming() {
    return this.tables.naming;
  }

  getTableName(entity) {
    const params = {
      classId: (0, _get2.default)(entity, "constructor.classId", (0, _get2.default)(entity, "classId")),
      storageClassId: (0, _get2.default)(entity, "constructor.storageClassId", (0, _get2.default)(entity, "storageClassId")),
      tableName: (0, _get2.default)(entity, "constructor.tableName", (0, _get2.default)(entity, "tableName"))
    };
    const getTableName = this.getTableNaming();

    if (typeof getTableName === "function") {
      return getTableName((0, _objectSpread2.default)({
        entity
      }, params, {
        driver: this
      }));
    }

    if (params.tableName) {
      return this.tables.prefix + params.tableName;
    }

    return this.tables.prefix + (params.storageClassId ? params.storageClassId : params.classId);
  }

  async test() {
    await this.getConnection().query("SELECT version()");
    return true;
  }

  static __preparePerPageOption(clonedOptions) {
    if ("perPage" in clonedOptions) {
      clonedOptions.limit = clonedOptions.perPage;
      delete clonedOptions.perPage;
    }
  }

  static __preparePageOption(clonedOptions) {
    if ("page" in clonedOptions) {
      clonedOptions.offset = clonedOptions.limit * (clonedOptions.page - 1);
      delete clonedOptions.page;
    }
  }

  static __prepareQueryOption(clonedOptions) {
    if (clonedOptions.query instanceof Object) {
      clonedOptions.where = clonedOptions.query;
      delete clonedOptions.query;
    }
  }

  static __prepareSearchOption(clonedOptions) {
    // Here we handle search (if passed) - we transform received arguments into linked LIKE statements.
    if (clonedOptions.search instanceof Object) {
      const {
        query,
        operator,
        fields: columns
      } = clonedOptions.search;
      const search = {
        $search: {
          operator,
          columns,
          query
        }
      };

      if (clonedOptions.where instanceof Object) {
        clonedOptions.where = {
          $and: [search, clonedOptions.where]
        };
      } else {
        clonedOptions.where = search;
      }

      delete clonedOptions.search;
    }
  }

  static __generateID() {
    return (0, _mdbid.default)();
  }

}

var _default = MySQLDriver;
exports.default = _default;
//# sourceMappingURL=mysqlDriver.js.map